@@ -42,6 +42,7 @@
 #include <openssl/sha.h>
 #endif // PODOFO_HAVE_LIBIDN
 
+#include <openssl/opensslconf.h>
 #include <openssl/md5.h>
 #include <openssl/evp.h>
 #endif //PODOFO_HAVE_OPENSSL
@@ -56,14 +57,18 @@ namespace PoDoFo {
 
 #ifdef PODOFO_HAVE_LIBIDN
 int PdfEncrypt::s_nEnabledEncryptionAlgorithms = 
+#ifndef OPENSSL_NO_RC4
 ePdfEncryptAlgorithm_RC4V1 |
 ePdfEncryptAlgorithm_RC4V2 |
+#endif // OPENSSL_NO_RC4
 ePdfEncryptAlgorithm_AESV2 |
 ePdfEncryptAlgorithm_AESV3;
 #else // PODOFO_HAVE_LIBIDN
 int PdfEncrypt::s_nEnabledEncryptionAlgorithms =
+#ifndef OPENSSL_NO_RC4
 ePdfEncryptAlgorithm_RC4V1 |
 ePdfEncryptAlgorithm_RC4V2 |
+#endif // OPENSSL_NO_RC4
 ePdfEncryptAlgorithm_AESV2;
 #endif // PODOFO_HAVE_LIBIDN
 
@@ -99,21 +104,41 @@ class AESCryptoEngine {
     
         AESCryptoEngine()
         {
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+            aes = EVP_CIPHER_CTX_new();
+    #else
             EVP_CIPHER_CTX_init(&aes);
+    #endif
         }
     
-        EVP_CIPHER_CTX* getEngine() {return &aes;}
+        EVP_CIPHER_CTX* getEngine()
+        {
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+            return aes;
+    #else
+            return &aes;
+    #endif
+        }
     
         ~AESCryptoEngine()
         {
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+            EVP_CIPHER_CTX_free(aes);
+    #else
             EVP_CIPHER_CTX_cleanup(&aes);
+    #endif
         }
     
     private:
-    
+
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+        EVP_CIPHER_CTX *aes;
+    #else
         EVP_CIPHER_CTX aes;
+    #endif
 };
 
+#ifndef OPENSSL_NO_RC4
 // A class that holds the RC4 Crypto object
 // Either CCCrpytor or EVP_CIPHER_CTX
 class RC4CryptoEngine {
@@ -121,19 +146,38 @@ public:
     
     RC4CryptoEngine()
     {
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+        rc4 = EVP_CIPHER_CTX_new();
+    #else
         EVP_CIPHER_CTX_init(&rc4);
+    #endif
     }
     
-    EVP_CIPHER_CTX* getEngine() {return &rc4;}
+    EVP_CIPHER_CTX* getEngine()
+    {
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+        return rc4;
+    #else
+        return &rc4;
+    #endif
+    }
     
     ~RC4CryptoEngine()
     {
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+        EVP_CIPHER_CTX_free(rc4);
+    #else
         EVP_CIPHER_CTX_cleanup(&rc4);
+    #endif
     }
     
 private:
     
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    EVP_CIPHER_CTX *rc4;
+    #else
     EVP_CIPHER_CTX rc4;
+    #endif
 };
     
 /** A class that can encrypt/decrpyt streamed data block wise
@@ -310,6 +354,7 @@ private:
     PdfInputStream* m_pInputStream;
     PdfRC4Stream    m_stream;
 };
+#endif // OPENSSL_NO_RC4
 
 /** A class that can encrypt/decrpyt streamed data block wise
  *  This is used in the input and output stream encryption implementation.
@@ -467,6 +512,7 @@ PdfEncrypt::CreatePdfEncrypt( const std::string & userPassword,
     switch (eAlgorithm)
     {
         case ePdfEncryptAlgorithm_AESV2:
+	default:
             pdfEncrypt = new PdfEncryptAESV2(userPassword, ownerPassword, protection);
             break;
 #ifdef PODOFO_HAVE_LIBIDN
@@ -474,11 +520,12 @@ PdfEncrypt::CreatePdfEncrypt( const std::string & userPassword,
             pdfEncrypt = new PdfEncryptAESV3(userPassword, ownerPassword, protection);
             break;
 #endif // PODOFO_HAVE_LIBIDN
+#ifndef OPENSSL_NO_RC4
         case ePdfEncryptAlgorithm_RC4V2:           
         case ePdfEncryptAlgorithm_RC4V1:
-        default:
             pdfEncrypt = new PdfEncryptRC4(userPassword, ownerPassword, protection, eAlgorithm, eKeyLength);
             break;
+#endif // OPENSSL_NO_RC4
     }
     return pdfEncrypt;
 }
@@ -550,6 +597,7 @@ PdfEncrypt* PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
         throw e;
     }
     
+#ifndef OPENSSL_NO_RC4
     if( (lV == 1L) && (rValue == 2L || rValue == 3L)
        && PdfEncrypt::IsEncryptionEnabled( ePdfEncryptAlgorithm_RC4V1 ) ) 
     {
@@ -561,7 +609,9 @@ PdfEncrypt* PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
         // [Alexey] - lLength is pdf_int64. Please make changes in encryption algorithms
         pdfEncrypt = new PdfEncryptRC4(oValue, uValue, pValue, rValue, ePdfEncryptAlgorithm_RC4V2, static_cast<int>(lLength), encryptMetadata);
     }
-    else if( (lV == 4L) && (rValue == 4L)
+    else 
+#endif // OPENSSL_NO_RC4
+    if( (lV == 4L) && (rValue == 4L)
             && PdfEncrypt::IsEncryptionEnabled( ePdfEncryptAlgorithm_AESV2 ) ) 
     {
         pdfEncrypt = new PdfEncryptAESV2(oValue, uValue, pValue, encryptMetadata);      
@@ -597,8 +647,10 @@ PdfEncrypt::CreatePdfEncrypt(const PdfEncrypt & rhs )
     else if (rhs.m_eAlgorithm == ePdfEncryptAlgorithm_AESV3)
         pdfEncrypt = new PdfEncryptAESV3(rhs);
 #endif // PODOFO_HAVE_LIBIDN
+#ifndef OPENSSL_NO_RC4
     else
         pdfEncrypt = new PdfEncryptRC4(rhs);
+#endif // OPENSSL_NO_RC4
     return pdfEncrypt;
 }
 
@@ -725,13 +777,17 @@ PdfEncryptMD5Base::ComputeOwnerKey(unsigned char userPad[32], unsigned char owne
                 else
                     mkey[j] = static_cast<unsigned char>(static_cast<unsigned int>(digest[j] ^ i));
             }
+#ifndef OPENSSL_NO_RC4
             RC4(mkey, keyLength, ownerKey, 32, ownerKey, 32);
+#endif // OPENSSL_NO_RC4
         }
     }
+#ifndef OPENSSL_NO_RC4
     else
     {
         RC4(digest, 5, userPad, 32, ownerKey, 32);
     }
+#endif // OPENSSL_NO_RC4
 }
 
 void
@@ -840,13 +896,17 @@ PdfEncryptMD5Base::ComputeEncryptionKey(const std::string& documentId,
                 digest[j] = static_cast<unsigned char>(m_encryptionKey[j] ^ k);
             }
             
+#ifndef OPENSSL_NO_RC4
             RC4(digest, m_keyLength, userKey, 16, userKey, 16);
+#endif // OPENSSL_NO_RC4
         }
     }
+#ifndef OPENSSL_NO_RC4
     else
     {
         RC4(m_encryptionKey, m_keyLength, padding, 32, userKey, 32);
     }
+#endif // OPENSSL_NO_RC4
     if (docId != NULL)
     {
         delete [] docId;
@@ -886,6 +946,7 @@ void PdfEncryptMD5Base::CreateObjKey( unsigned char objkey[16], int* pnKeyLen )
     *pnKeyLen = (m_keyLength <= 11) ? m_keyLength+5 : 16;
 }
 
+#ifndef OPENSSL_NO_RC4
 PdfEncryptRC4Base::PdfEncryptRC4Base()
 {
     m_rc4 = new RC4CryptoEngine();
@@ -933,6 +994,7 @@ PdfEncryptRC4Base::RC4(const unsigned char* key, int keylen,
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error RC4-encrypting data" );
 }
+#endif // OPENSSL_NO_RC4
         
 void
 PdfEncryptMD5Base::GetMD5Binary(const unsigned char* data, int length, unsigned char* digest)
@@ -974,9 +1036,11 @@ void PdfEncryptMD5Base::CreateEncryptionDictionary( PdfDictionary & rDictionary
         PdfDictionary cf;
         PdfDictionary stdCf;
         
+#ifndef OPENSSL_NO_RC4
 		if(m_eAlgorithm == ePdfEncryptAlgorithm_RC4V2)
 			stdCf.AddKey( PdfName("CFM"), PdfName("V2") );
 		else
+#endif // OPENSSL_NO_RC4
 			stdCf.AddKey( PdfName("CFM"), PdfName("AESV2") );
         stdCf.AddKey( PdfName("Length"), static_cast<pdf_int64>(PODOFO_LL_LITERAL(16)) );
         
@@ -995,7 +1059,8 @@ void PdfEncryptMD5Base::CreateEncryptionDictionary( PdfDictionary & rDictionary
         rDictionary.AddKey( PdfName("Length"), static_cast<pdf_int64>(PODOFO_LL_LITERAL(128)) );
 		if(!m_bEncryptMetadata)
 			rDictionary.AddKey( PdfName("EncryptMetadata"), PdfVariant( false ) );
-	}
+    }
+#ifndef OPENSSL_NO_RC4
     else if(m_eAlgorithm == ePdfEncryptAlgorithm_RC4V1)
     {
         rDictionary.AddKey( PdfName("V"), static_cast<pdf_int64>(PODOFO_LL_LITERAL(1)) );
@@ -1008,12 +1073,14 @@ void PdfEncryptMD5Base::CreateEncryptionDictionary( PdfDictionary & rDictionary
         rDictionary.AddKey( PdfName("R"), static_cast<pdf_int64>(PODOFO_LL_LITERAL(3)) );
 		rDictionary.AddKey( PdfName("Length"), PdfVariant( static_cast<pdf_int64>(m_eKeyLength) ) );
     }
+#endif // OPENSSL_NO_RC4
     
     rDictionary.AddKey( PdfName("O"), PdfString( reinterpret_cast<const char*>(this->GetOValue()), 32, true ) );
     rDictionary.AddKey( PdfName("U"), PdfString( reinterpret_cast<const char*>(this->GetUValue()), 32, true ) );
     rDictionary.AddKey( PdfName("P"), PdfVariant( static_cast<pdf_int64>(this->GetPValue()) ) );
 }
     
+#ifndef OPENSSL_NO_RC4
 void
 PdfEncryptRC4::GenerateEncryptionKey(const PdfString & documentId)
 {
@@ -1172,6 +1239,7 @@ PdfOutputStream* PdfEncryptRC4::CreateEncryptionOutputStream( PdfOutputStream* p
     
     return new PdfRC4OutputStream( pOutputStream, m_rc4key, m_rc4last, objkey, keylen );
 }
+#endif // OPENSSL_NO_RC4
     
 PdfEncryptAESBase::PdfEncryptAESBase()
 {
@@ -1459,24 +1527,34 @@ void PdfEncryptSHABase::ComputeUserKey(const unsigned char * userpswd, int len)
     // UE = AES-256 encoded file encryption key with key=hash
     // CBC mode, no padding, init vector=0
     
-    EVP_CIPHER_CTX aes;
-    EVP_CIPHER_CTX_init(&aes);
+    EVP_CIPHER_CTX *aes;
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    aes = EVP_CIPHER_CTX_new();
+    #else
+    EVP_CIPHER_CTX aes_local;
+    EVP_CIPHER_CTX_init(&aes_local);
+    aes = &aes_local;
+    #endif
     
-    int status = EVP_EncryptInit_ex(&aes, EVP_aes_256_cbc(), NULL, hashValue, NULL);
+    int status = EVP_EncryptInit_ex(aes, EVP_aes_256_cbc(), NULL, hashValue, NULL);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error initializing AES encryption engine" );
-    EVP_CIPHER_CTX_set_padding(&aes, 0); // disable padding
+    EVP_CIPHER_CTX_set_padding(aes, 0); // disable padding
     
     int dataOutMoved;
-    status = EVP_EncryptUpdate(&aes, m_ueValue, &dataOutMoved, m_encryptionKey, m_keyLength);
+    status = EVP_EncryptUpdate(aes, m_ueValue, &dataOutMoved, m_encryptionKey, m_keyLength);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error AES-encrypting data" );
     
-    status = EVP_EncryptFinal_ex(&aes, &m_ueValue[dataOutMoved], &dataOutMoved);
+    status = EVP_EncryptFinal_ex(aes, &m_ueValue[dataOutMoved], &dataOutMoved);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error AES-encrypting data" );
     
-    EVP_CIPHER_CTX_cleanup(&aes);
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    EVP_CIPHER_CTX_free(aes);
+    #else
+    EVP_CIPHER_CTX_cleanup(&aes_local);
+    #endif
 }
 
 void PdfEncryptSHABase::ComputeOwnerKey(const unsigned char * ownerpswd, int len)
@@ -1515,24 +1593,34 @@ void PdfEncryptSHABase::ComputeOwnerKey(const unsigned char * ownerpswd, int len
     // OE = AES-256 encoded file encryption key with key=hash
     // CBC mode, no padding, init vector=0
     
-    EVP_CIPHER_CTX aes;
-    EVP_CIPHER_CTX_init(&aes);
+    EVP_CIPHER_CTX *aes;
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    aes = EVP_CIPHER_CTX_new();
+    #else
+    EVP_CIPHER_CTX aes_local;
+    EVP_CIPHER_CTX_init(&aes_local);
+    aes = &aes_local;
+    #endif
     
-    int status = EVP_EncryptInit_ex(&aes, EVP_aes_256_cbc(), NULL, hashValue, NULL);
+    int status = EVP_EncryptInit_ex(aes, EVP_aes_256_cbc(), NULL, hashValue, NULL);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error initializing AES encryption engine" );
-    EVP_CIPHER_CTX_set_padding(&aes, 0); // disable padding
+    EVP_CIPHER_CTX_set_padding(aes, 0); // disable padding
     
     int dataOutMoved;
-    status = EVP_EncryptUpdate(&aes, m_oeValue, &dataOutMoved, m_encryptionKey, m_keyLength);
+    status = EVP_EncryptUpdate(aes, m_oeValue, &dataOutMoved, m_encryptionKey, m_keyLength);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error AES-encrypting data" );
     
-    status = EVP_EncryptFinal_ex(&aes, &m_oeValue[dataOutMoved], &dataOutMoved);
+    status = EVP_EncryptFinal_ex(aes, &m_oeValue[dataOutMoved], &dataOutMoved);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error AES-encrypting data" );
     
-    EVP_CIPHER_CTX_cleanup(&aes);
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    EVP_CIPHER_CTX_free(aes);
+    #else
+    EVP_CIPHER_CTX_cleanup(&aes_local);
+    #endif
 }
 
 void PdfEncryptSHABase::PreprocessPassword( const std::string &password, unsigned char* outBuf, int &len)
@@ -1663,24 +1751,34 @@ PdfEncryptAESV3::GenerateEncryptionKey(const PdfString &)
     
     // Encrypt Perms value
     
-    EVP_CIPHER_CTX aes;
-    EVP_CIPHER_CTX_init(&aes);
+    EVP_CIPHER_CTX *aes;
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    aes = EVP_CIPHER_CTX_new();
+    #else
+    EVP_CIPHER_CTX aes_local;
+    EVP_CIPHER_CTX_init(&aes_local);
+    aes = &aes_local;
+    #endif
     
-    int status = EVP_EncryptInit_ex(&aes, EVP_aes_256_ecb(), NULL, m_encryptionKey, NULL);
+    int status = EVP_EncryptInit_ex(aes, EVP_aes_256_ecb(), NULL, m_encryptionKey, NULL);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error initializing AES encryption engine" );
-    EVP_CIPHER_CTX_set_padding(&aes, 0); // disable padding
+    EVP_CIPHER_CTX_set_padding(aes, 0); // disable padding
     
     int dataOutMoved;
-    status = EVP_EncryptUpdate(&aes, m_permsValue, &dataOutMoved, perms, 16);
+    status = EVP_EncryptUpdate(aes, m_permsValue, &dataOutMoved, perms, 16);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error AES-encrypting data" );
     
-    status = EVP_EncryptFinal_ex(&aes, &m_permsValue[dataOutMoved], &dataOutMoved);
+    status = EVP_EncryptFinal_ex(aes, &m_permsValue[dataOutMoved], &dataOutMoved);
     if(status != 1)
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Error AES-encrypting data" );
     
-    EVP_CIPHER_CTX_cleanup(&aes);
+    #ifdef PODOFO_HAVE_OPENSSL_1_1
+    EVP_CIPHER_CTX_free(aes);
+    #else
+    EVP_CIPHER_CTX_cleanup(&aes_local);
+    #endif
 }
 
 bool PdfEncryptAESV3::Authenticate( const std::string & password, const PdfString & )
@@ -1845,7 +1943,7 @@ PdfOutputStream* PdfEncryptAESV3::CreateEncryptionOutputStream( PdfOutputStream*
 #endif // PODOFO_HAVE_LIBIDN
 #else  // PODOFO_HAVE_OPENSSL
 // ----------------
-// MD5 by RSA
+// MD4 by RSA
 // ----------------
 
 // C headers for MD5
