@@ -154,7 +154,7 @@ void PdfError::PrintErrorMsg() const
 
     int i                = 0;
 
-    PdfError::LogErrorMessage( eLogSeverity_Error, "\n\nPoDoFo encounter an error. Error: %i %s\n", m_error, pszName ? pszName : "" );
+    PdfError::LogErrorMessage( eLogSeverity_Error, "\n\nPoDoFo encountered an error. Error: %i %s\n", m_error, pszName ? pszName : "" );
 
     if( pszMsg )
         PdfError::LogErrorMessage( eLogSeverity_Error, "\tError Description: %s\n", pszMsg );
@@ -342,6 +342,15 @@ const char* PdfError::ErrorName( EPdfError eCode )
         case ePdfError_ChangeOnImmutable:
             pszMsg = "ePdfError_ChangeOnImmutable";
             break;
+        case ePdfError_OutlineItemAlreadyPresent:
+            pszMsg = "ePdfError_OutlineItemAlreadyPresent"; 
+            break;
+        case ePdfError_NotLoadedForUpdate:
+            pszMsg = "ePdfError_NotLoadedForUpdate"; 
+            break;
+        case ePdfError_CannotEncryptedForUpdate:
+            pszMsg = "ePdfError_CannotEncryptedForUpdate"; 
+            break;
         case ePdfError_Unknown:
             pszMsg = "ePdfError_Unknown"; 
             break;
@@ -478,6 +487,15 @@ const char* PdfError::ErrorMessage( EPdfError eCode )
         case ePdfError_NotCompiled:
             pszMsg = "This feature was disabled during compile time.";
             break;
+        case ePdfError_OutlineItemAlreadyPresent:
+            pszMsg = "Given OutlineItem already present in destination tree.";
+            break;
+        case ePdfError_NotLoadedForUpdate:
+            pszMsg = "The document had not been loaded for update.";
+            break;
+        case ePdfError_CannotEncryptedForUpdate:
+            pszMsg = "Cannot load encrypted documents for update.";
+            break;
         case ePdfError_Unknown:
             pszMsg = "Error code unknown.";
             break;
@@ -581,6 +599,16 @@ void PdfError::LogMessage( ELogSeverity eLogSeverity, const wchar_t* pszMsg, ...
     va_end( args );
 }
 
+void PdfError::EnableLogging( bool bEnable )
+{
+    PdfError::s_LogEnabled = bEnable;
+}
+
+bool PdfError::LoggingEnabled()
+{
+    return PdfError::s_LogEnabled;
+}
+
 void PdfError::LogErrorMessage( ELogSeverity eLogSeverity, const wchar_t* pszMsg, ... )
 {
     va_list  args;
@@ -642,14 +670,27 @@ void PdfError::DebugMessage( const char* pszMsg, ... )
     if ( m_fLogMessageCallback != NULL )
     {
         m_fLogMessageCallback->LogMessage(eLogSeverity_Debug, pszPrefix, pszMsg, args);
-        return;
+    }
+    else
+    {
+        if( pszPrefix )
+            fprintf( stderr, "%s", pszPrefix );
+
+        vfprintf( stderr, pszMsg, args );
     }
 
-	if( pszPrefix )
-		fprintf( stderr, "%s", pszPrefix );
-
-	vfprintf( stderr, pszMsg, args );
+    // must call va_end after calling va_start (which allocates memory on some platforms)
 	va_end( args );
 }
 
+void PdfError::EnableDebug( bool bEnable )
+{
+	PdfError::s_DgbEnabled = bEnable;
+}
+
+bool PdfError::DebugEnabled()
+{
+	return PdfError::s_DgbEnabled;
+}
+
 };
