@@ -101,7 +101,7 @@ private:
 };
 
 PdfVecObjects::PdfVecObjects()
-    : m_bAutoDelete( false ), m_nObjectCount( 1 ), m_bSorted( true ), m_pDocument( NULL ), m_pStreamFactory( NULL )
+    : m_bAutoDelete( false ), m_bCanReuseObjectNumbers( true ), m_nObjectCount( 1 ), m_bSorted( true ), m_pDocument( NULL ), m_pStreamFactory( NULL )
 {
 }
 
@@ -225,7 +225,7 @@ PdfReference PdfVecObjects::GetNextFreeObject()
 {
     PdfReference ref( static_cast<unsigned int>(m_nObjectCount), 0 );
 
-    if( !m_lstFreeObjects.empty() )
+    if( m_bCanReuseObjectNumbers && !m_lstFreeObjects.empty() )
     {
         ref = m_lstFreeObjects.front();
         m_lstFreeObjects.pop_front();
@@ -264,7 +264,7 @@ void PdfVecObjects::AddFreeObject( const PdfReference & rReference )
     if( it.first != it.second && !m_lstFreeObjects.empty() ) 
     {
         // Be sure that no reference is added twice to free list
-        PdfError::DebugMessage( "Adding %d to freelist, is already contained !!", rReference.ObjectNumber() );
+        PdfError::DebugMessage( "Adding %d to free list, is already contained in it!", rReference.ObjectNumber() );
         return;
     }
     else
@@ -591,7 +591,7 @@ std::string PdfVecObjects::GetNextSubsetPrefix()
 	else
 	{
 		PODOFO_ASSERT( m_sSubsetPrefix.length() == 7 );
-		PODOFO_ASSERT( m_sSubsetPrefix[6] = '+' );
+		PODOFO_ASSERT( m_sSubsetPrefix[6] == '+' );
 	
 		for ( int i = 5; i >= 0; i-- )
 		{
@@ -607,6 +607,16 @@ std::string PdfVecObjects::GetNextSubsetPrefix()
 	return m_sSubsetPrefix;
 }
 
+void PdfVecObjects::SetCanReuseObjectNumbers( bool bCanReuseObjectNumbers )
+{
+    m_bCanReuseObjectNumbers = bCanReuseObjectNumbers;
+
+    if( !m_bCanReuseObjectNumbers )
+    {
+        m_lstFreeObjects.clear();
+    }
+}
+
 };
 
 
