@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfField.h"
@@ -69,6 +82,31 @@ PdfField::PdfField( EPdfField eField, PdfPage* pPage, const PdfRect & rRect, Pdf
     Init( pDoc->GetAcroForm() );
 }
 
+PdfField::PdfField( EPdfField eField, PdfAnnotation* pWidget, PdfAcroForm* pParent, PdfDocument* pDoc)
+    : m_pObject( pWidget->GetObject() ), m_pWidget( pWidget ), m_eField( eField )
+{
+    Init( pParent );
+    PdfObject* pFields = pParent->GetObject()->GetDictionary().GetKey( PdfName("Fields") );
+    if( pFields && pFields->IsReference())  {
+       PdfObject *pRefFld = pDoc->GetObjects()->GetObject(pFields->GetReference());
+       if(pRefFld)
+         pRefFld->GetArray().push_back( m_pObject->Reference() );
+    }
+}
+
+PdfField::PdfField( EPdfField eField, PdfPage* pPage, const PdfRect & rRect, PdfDocument* pDoc, bool bAppearanceNone)
+    :  m_eField( eField )
+{
+   m_pWidget = pPage->CreateAnnotation( ePdfAnnotation_Widget, rRect );
+   m_pObject = m_pWidget->GetObject();
+
+   Init( 
+	pDoc->GetAcroForm(true, 
+			  bAppearanceNone ? 
+			  ePdfAcroFormDefaultAppearance_None
+			  : ePdfAcroFormDefaultAppearance_BlackText12pt ));
+}
+
 PdfField::PdfField( const PdfField & rhs )
     : m_pObject( NULL ), m_pWidget( NULL ), m_eField( ePdfField_Unknown )
 {
@@ -81,7 +119,10 @@ void PdfField::Init( PdfAcroForm* pParent )
     PdfObject* pFields = pParent->GetObject()->GetDictionary().GetKey( PdfName("Fields") );
     if( pFields ) 
     {
-        pFields->GetArray().push_back( m_pObject->Reference() );
+        if(!pFields->IsReference() ) 
+	{
+            pFields->GetArray().push_back( m_pObject->Reference() );
+	}
     }
     else
     {
@@ -115,9 +156,6 @@ void PdfField::Init( PdfAcroForm* pParent )
         break;
     }
 
-
-    m_pWidget->SetBorderStyle( 0.0, 0.0, 5.0 );
-
     // Create a unique fieldname, because Acrobat Reader crashes if the field has no field name 
     std::ostringstream out;
     PdfLocaleImbue(out);
@@ -127,8 +165,26 @@ void PdfField::Init( PdfAcroForm* pParent )
 PdfField::PdfField( PdfObject* pObject, PdfAnnotation* pWidget )
     : m_pObject( pObject ), m_pWidget( pWidget ), m_eField( ePdfField_Unknown )
 {
+    // ISO 32000:2008, Section 12.7.3.1, Table 220, Page #432.
+    const PdfObject *pFT = m_pObject->GetDictionary().GetKey(PdfName("FT") );
 
-    PdfName fieldType = m_pObject->GetDictionary().GetKey( PdfName("FT") )->GetName();
+    if (!pFT && m_pObject->GetDictionary().HasKey( PdfName ("Parent") ) )
+    {
+        const PdfObject *pTemp =  m_pObject->GetIndirectKey ( PdfName("Parent") );
+        if (!pTemp)
+        {
+            PODOFO_RAISE_ERROR (ePdfError_InvalidDataType);
+        }
+
+        pFT = pTemp->GetDictionary().GetKey( PdfName ("FT") );
+    }
+
+    if (!pFT)
+    {
+        PODOFO_RAISE_ERROR (ePdfError_NoObject);
+    }
+
+    const PdfName fieldType = pFT->GetName();
 
     if( fieldType == PdfName("Btn") )
     {
@@ -187,7 +243,7 @@ void PdfField::SetFieldFlag( long lValue, bool bSet )
 
 bool PdfField::GetFieldFlag( long lValue, bool bDefault ) const
 {
-    long long lCur = 0;
+    pdf_int64 lCur = 0;
 
     if( m_pObject->GetDictionary().HasKey( PdfName("Ff") ) )
     {
@@ -635,9 +691,17 @@ void PdfCheckBox::SetChecked( bool bChecked )
 
 bool PdfCheckBox::IsChecked() const
 {
-    PdfName name = m_pObject->GetDictionary().GetKey( PdfName("V") )->GetName();
+    PdfDictionary dic = m_pObject->GetDictionary();
 
-    return name == PdfName("Yes");
+    if (dic.HasKey(PdfName("V"))) {
+        PdfName name = dic.GetKey( PdfName("V") )->GetName();
+        return (name == PdfName("Yes") || name == PdfName("On"));
+     } else if (dic.HasKey(PdfName("AS"))) {
+        PdfName name = dic.GetKey( PdfName("AS") )->GetName();
+        return (name == PdfName("Yes") || name == PdfName("On"));
+     }
+
+    return false;
 }
 
 /////////////////////////////////////////////////////////////////////////////
