@@ -52,8 +52,8 @@
 #ifdef _WIN32
 
 //#include <windows.h>
-// Undefined stuff which windows does
-// define that breaks are build
+// Undefined stuff which Windows does
+// define that breaks our build
 // e.g. GetObject is defined to either GetObjectA or GetObjectW
 #ifdef GetObject
 #undef GetObject
@@ -104,135 +104,97 @@ inline unsigned short ShortFromBigEndian(unsigned short i)
 namespace PoDoFo {
 
 #if defined(_WIN32) && !defined(PODOFO_NO_FONTMANAGER)
-// The function receives a buffer containing a true type collection and replaces the buffer
-// by a new buffer with the extracted font.
-// On error the function returns false.
-static bool GetFontFromCollection(char *&buffer, unsigned int &bufferLen, unsigned int bufferOffset, const LOGFONTW* inFont)
+//This function will recieve the device context for the ttc font, it will then extract necessary tables,and create the correct buffer.
+//On error function return false
+static bool GetFontFromCollection(HDC &hdc, char *&buffer, unsigned int &bufferLen)
 {
-    bool ok = false;
+    const DWORD ttcf_const = 0x66637474;
+    unsigned int fileLen = GetFontData(hdc, ttcf_const, 0, 0, 0);
+    unsigned int ttcLen = GetFontData(hdc, 0, 0, 0, 0);
+    if (fileLen == GDI_ERROR || ttcLen == GDI_ERROR)
+    {
+        return false;
+    }
 
-    // these properties are extracted to match the font
-    wchar_t fontFamily[1024];
-    wchar_t fontStyle[1024];
-    wchar_t fontFullName[1024];
-    wchar_t fontPostscriptName[1024];
-    fontFamily[0]   = 0;
-    fontStyle[0]    = 0;
-    fontFullName[0] = 0;
-    fontPostscriptName[0] = 0;
-    unsigned int fontFileSize = 12;
+    char *fileBuffer = (char*)podofo_malloc(fileLen);
+    if (!fileBuffer)
+    {
+        PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+    }
+
+    char *ttcBuffer = (char*)podofo_malloc(ttcLen);
+    if (!ttcBuffer)
+    {
+        podofo_free(fileBuffer);
+        PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+    }
+
+    if (GetFontData(hdc, ttcf_const, 0, fileBuffer, fileLen) == GDI_ERROR)
+    {
+        podofo_free(fileBuffer);
+        podofo_free(ttcBuffer);
+        return false;
+    }
+
+    if (GetFontData(hdc, 0, 0, ttcBuffer, ttcLen) == GDI_ERROR)
+    {
+        podofo_free(fileBuffer);
+        podofo_free(ttcBuffer);
+        return false;
+    }
+
+    USHORT numTables = ShortFromBigEndian(*(USHORT *)(ttcBuffer + 4));
+    unsigned int outLen = 12+16* numTables;
+    char *entry = ttcBuffer + 12;
+    int table;
 
     //us: see "http://www.microsoft.com/typography/otspec/otff.htm"
-    USHORT numTables = ShortFromBigEndian(*(USHORT *)(buffer + bufferOffset + 4));
-    char *entry = buffer + bufferOffset + 12;
-    for(int i=0; i<numTables; i++)
+    for (table = 0; table < numTables; table++)
     {
-        char tag[5];
-        tag[0] = entry[0];
-        tag[1] = entry[1];
-        tag[2] = entry[2];
-        tag[3] = entry[3];
-        tag[4] = 0;
-        ULONG checkSum = FromBigEndian(*(ULONG *)(entry+4));
-        ULONG offset   = FromBigEndian(*(ULONG *)(entry+8));
-        ULONG length   = FromBigEndian(*(ULONG *)(entry+12));
-        length = (length+3) & ~3;
-        if(offset+length > bufferLen) return false; // truncated or corrupted buffer
-        if(strcmp(tag, "name") == 0)
-        {
-            //us: see "http://www.microsoft.com/typography/otspec/name.htm"
-            char *nameTable = buffer + offset;
-            USHORT format       = ShortFromBigEndian(*(USHORT *)(nameTable));
-            USHORT nameCount    = ShortFromBigEndian(*(USHORT *)(nameTable+2));
-            USHORT stringOffset = ShortFromBigEndian(*(USHORT *)(nameTable+4));
-            char *stringArea = nameTable + stringOffset;
-            char *nameRecord = nameTable + 6;
-            for(int n=0; n<nameCount; n++)
-            {
-                USHORT platformID = ShortFromBigEndian(*(USHORT *)(nameRecord));
-                USHORT encodingID = ShortFromBigEndian(*(USHORT *)(nameRecord+2));
-                USHORT languageID = ShortFromBigEndian(*(USHORT *)(nameRecord+4));
-                if(platformID == 0 && languageID == 0)
-                {
-                    // only Unicode platform / english
-                    USHORT nameID     = ShortFromBigEndian(*(USHORT *)(nameRecord+6));
-                    USHORT length     = ShortFromBigEndian(*(USHORT *)(nameRecord+8));
-                    USHORT offset     = ShortFromBigEndian(*(USHORT *)(nameRecord+10));
-                    wchar_t name[1024];
-                    if(length >= sizeof(name)) length = sizeof(name) - sizeof(wchar_t);
-                    unsigned int charCount = length / sizeof(wchar_t);
-                    for(unsigned int i=0; i<charCount; i++)
-                    {
-                        name[i] = ShortFromBigEndian(((USHORT *)(stringArea + offset))[i]);
-                    }
-                    name[charCount] = 0;
-                    switch(nameID)
-                    {
-                    case 1:
-                        wcscpy(fontFamily, name);
-                        break;
-                    case 2:
-                        wcscpy(fontStyle, name);
-                        break;
-                    case 4:
-                        wcscpy(fontFullName, name);
-                        break;
-                    case 6:
-                        wcscpy(fontPostscriptName, name);
-                        break;
-                    }
-                }
-                nameRecord += 12;
-            }
-        }
+        ULONG length = FromBigEndian(*(ULONG *)(entry + 12));
+        length = (length + 3) & ~3;
         entry += 16;
-        fontFileSize += 16 + length;
+        outLen += length;
+    }
+    char *outBuffer = (char*)podofo_malloc(outLen);
+    if (!outBuffer)
+    {
+        podofo_free(fileBuffer);
+        podofo_free(ttcBuffer);
+        PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+    }
+    // copy font header and table index (offsets need to be still adjusted)
+    memcpy(outBuffer, ttcBuffer, 12 + 16 * numTables);
+    unsigned int dstDataOffset = 12 + 16 * numTables;
+
+    // process tables
+    char *srcEntry = ttcBuffer + 12;
+    char *dstEntry = outBuffer + 12;
+    for (table = 0; table < numTables; table++)
+    {
+        // read source entry
+        ULONG offset = FromBigEndian(*(ULONG *)(srcEntry + 8));
+        ULONG length = FromBigEndian(*(ULONG *)(srcEntry + 12));
+        length = (length + 3) & ~3;
+
+        // adjust offset
+        // U can use FromBigEndian() also to convert _to_ big endian
+        *(ULONG *)(dstEntry + 8) = FromBigEndian(dstDataOffset);
+
+        //copy data
+        memcpy(outBuffer + dstDataOffset, fileBuffer + offset, length);
+        dstDataOffset += length;
+
+        // adjust table entry pointers for loop
+        srcEntry += 16;
+        dstEntry += 16;
     }
 
-    // check if font matches the wanted properties
-    unsigned int isMatchingFont = _wcsicmp(fontFamily, inFont->lfFaceName) == 0;
-    //..TODO.. check additional styles (check fontStyle, fontFullName, fontPostscriptName) ?
-
-    // extract font
-    if(isMatchingFont) {
-        char *newBuffer = (char *) malloc(fontFileSize);
-
-        // copy font header and table index (offsets need to be still adjusted)
-        memcpy(newBuffer, buffer + bufferOffset, 12+16*numTables);
-        unsigned int dstDataOffset = 12+16*numTables;
-
-        // process tables
-        char *srcEntry = buffer + bufferOffset + 12;
-        char *dstEntry = newBuffer + 12;
-        for(int table=0; table < numTables; table++)
-        {
-            // read source entry
-            ULONG offset   = FromBigEndian(*(ULONG *)(srcEntry+8));
-            ULONG length   = FromBigEndian(*(ULONG *)(srcEntry+12));
-            length = (length+3) & ~3;
-
-            // adjust offset
-            // U can use FromBigEndian() also to convert _to_ big endian 
-            *(ULONG *)(dstEntry+8) = FromBigEndian(dstDataOffset);
-
-            //copy data
-            memcpy(newBuffer + dstDataOffset, buffer + offset, length);
-            dstDataOffset += length;
-
-            // adjust table entry pointers for loop
-            srcEntry += 16;
-            dstEntry += 16;
-        }
-
-        // replace old buffer
-        //assert(dstDataOffset==fontFileSize)
-        free(buffer);
-        buffer = newBuffer;
-        bufferLen = fontFileSize;
-        ok = true;
-    }
-
-    return ok;
+    podofo_free(fileBuffer);
+    podofo_free(ttcBuffer);
+    buffer = outBuffer;
+    bufferLen = outLen;
+    return true;
 }
 
 static bool GetDataFromHFONT( HFONT hf, char** outFontBuffer, unsigned int& outFontBufferLen, const LOGFONTW* inFont )
@@ -245,40 +207,23 @@ static bool GetDataFromHFONT( HFONT hf, char** outFontBuffer, unsigned int& outF
 
     // try get data from true type collection
     char *buffer = NULL;
-    unsigned int bufferLen = 0;
-    bool hasData = false;
     const DWORD ttcf_const = 0x66637474;
-    DWORD dwTable = ttcf_const;
-    bufferLen = GetFontData(hdc, dwTable, 0, 0, 0);
-    if (bufferLen == GDI_ERROR)
-    {
-        dwTable = 0;
-        bufferLen = GetFontData(hdc, dwTable, 0, 0, 0);
-    }
-    if (bufferLen != GDI_ERROR)
-    {
-        buffer = (char *) malloc( bufferLen );
-        hasData = GetFontData( hdc, dwTable, 0, buffer, (DWORD) bufferLen ) != GDI_ERROR;
-    }
+    unsigned int bufferLen = GetFontData(hdc, 0, 0, 0, 0);
+    unsigned int ttcLen = GetFontData(hdc, ttcf_const, 0, 0, 0);
 
-    if(hasData)
+	if (bufferLen != GDI_ERROR && ttcLen == GDI_ERROR)
     {
-        if(((DWORD *)buffer)[0] == ttcf_const)
-        {
-            // true type collection data
-            unsigned int numFonts = FromBigEndian(((unsigned int *)buffer)[2]);
-            for(unsigned int i=0; i<numFonts; i++)
-            {
-                unsigned int offset = FromBigEndian(((unsigned int *)buffer)[3+i]);
-                ok = GetFontFromCollection(buffer, bufferLen, offset, inFont);
-                if(ok) break;
-            }
-        }
-        else
-        {
-            // "normal" font data
-            ok = true;
-        }
+        buffer = (char *) podofo_malloc( bufferLen );
+		if (!buffer)
+		{
+			PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+		}
+
+        ok = GetFontData(hdc, 0, 0, buffer, (DWORD)bufferLen) != GDI_ERROR;
+    }
+	else if (bufferLen != GDI_ERROR)
+    {
+        ok = GetFontFromCollection(hdc, buffer, bufferLen);
     }
 
     // clean up
@@ -293,7 +238,7 @@ static bool GetDataFromHFONT( HFONT hf, char** outFontBuffer, unsigned int& outF
     else if(buffer)
     {
         // on failure free local buffer
-        free(buffer);
+        podofo_free(buffer);
     }
     return ok;
 }
@@ -423,8 +368,8 @@ PdfFont* PdfFontCache::GetFont( PdfObject* pObject )
         element.m_bItalic   = pFont->IsItalic();
         element.m_sFontName = pFont->GetFontMetrics()->GetFontname();
         element.m_pEncoding = NULL;
-		  element.m_bIsSymbolCharset = pFont->GetFontMetrics()->IsSymbol();
-        m_vecFonts  .push_back( element );
+        element.m_bIsSymbolCharset = pFont->GetFontMetrics()->IsSymbol();
+        m_vecFonts.push_back( element );
         
         // Now sort the font list
         std::sort( m_vecFonts.begin(), m_vecFonts.end() );
@@ -445,72 +390,79 @@ PdfFont* PdfFontCache::GetFont( const char* pszFontName, bool bBold, bool bItali
     std::pair<TISortedFontList,TCISortedFontList> it;
 
     it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), 
-			   TFontCacheElement( pszFontName, bBold, bItalic, bSymbolCharset, pEncoding ) );
+               TFontCacheElement( pszFontName, bBold, bItalic, bSymbolCharset, pEncoding ) );
 
-		
+        
     if( it.first == it.second )
     {
-		if ( (eFontCreationFlags & eFontCreationFlags_AutoSelectBase14) 
-             && PODOFO_Base14FontDef_FindBuiltinData(pszFontName))
-		{  
-         int nFlags = ePdfFont_Normal;
+        if ( (eFontCreationFlags & eFontCreationFlags_AutoSelectBase14) 
+             && PODOFO_Base14FontDef_FindBuiltinData(pszFontName) )
+        {
+            EPdfFontFlags eFlags = ePdfFont_Normal;
+            if( bBold )
+            {
+                if( bItalic )
+                {
+                    eFlags = ePdfFont_BoldItalic;
+                }
+                else
+                {
+                    eFlags = ePdfFont_Bold;
+                }
+            }
+            else if( bItalic )
+                eFlags = ePdfFont_Italic;
 
-         if( bBold ) 
-             nFlags |= ePdfFont_Bold;
+            pFont = PdfFontFactory::CreateBase14Font(pszFontName, eFlags,
+                        pEncoding, m_pParent);
+            if( pFont ) 
+            {
+                TFontCacheElement element;
+                element.m_pFont     = pFont;
+                element.m_bBold     = pFont->IsBold();
+                element.m_bItalic   = pFont->IsItalic();
+                element.m_sFontName = pszFontName;
+                element.m_pEncoding = pEncoding;
+                element.m_bIsSymbolCharset = bSymbolCharset;
 
-         if( bItalic )
-             nFlags |= ePdfFont_Italic;
+                // Do a sorted insert, so no need to sort again
+                //rvecContainer.insert( itSorted, element ); 
+                m_vecFonts.insert( it.first, element );
+                
+             }
 
-         pFont = PdfFontFactory::CreateBase14Font(pszFontName, nFlags, pEncoding, m_pParent);
+        }
 
-			if( pFont ) 
-			{
-				TFontCacheElement element;
-				element.m_pFont     = pFont;
-				element.m_bBold     = pFont->IsBold();
-				element.m_bItalic   = pFont->IsItalic();
-				element.m_sFontName = pszFontName;
-				element.m_pEncoding = pEncoding;
-				element.m_bIsSymbolCharset = bSymbolCharset;
-
-				// Do a sorted insert, so no need to sort again
-				//rvecContainer.insert( itSorted, element ); 
-				m_vecFonts.insert( it.first, element );
-				
-			 }
-
-		}
-
-		if (!pFont)
-		{
-			std::string sPath;
-			if ( pszFileName == NULL )
-				sPath = this->GetFontPath( pszFontName, bBold, bItalic );
-			else
-				sPath = pszFileName;
-	        
-			if( sPath.empty() )
-			{
+        if (!pFont)
+        {
+            bool bSubsetting = (eFontCreationFlags & eFontCreationFlags_Type1Subsetting) != 0;
+            std::string sPath;
+            if ( pszFileName == NULL )
+                sPath = this->GetFontPath( pszFontName, bBold, bItalic );
+            else
+                sPath = pszFileName;
+            
+            if( sPath.empty() )
+            {
 #if defined(_WIN32) && !defined(PODOFO_NO_FONTMANAGER)
-				pFont = GetWin32Font( it.first, m_vecFonts, pszFontName, bBold, bItalic, bSymbolCharset, bEmbedd, pEncoding );
+                pFont = GetWin32Font( it.first, m_vecFonts, pszFontName, bBold, bItalic, bSymbolCharset, bEmbedd, pEncoding, bSubsetting  );
 #endif // _WIN32
-			}
-			else
-			{
-				bool bSubsetting = (eFontCreationFlags & eFontCreationFlags_Type1Subsetting) != 0;
-				pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, sPath.c_str(), bSymbolCharset, bSubsetting ? genSubsetBasename() : NULL );
-				pFont    = this->CreateFontObject( it.first, m_vecFonts, pMetrics, 
-						   bEmbedd, bBold, bItalic, pszFontName, pEncoding, bSubsetting );
-			}
+            }
+            else
+            {
+                pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, sPath.c_str(), bSymbolCharset, bSubsetting ? genSubsetBasename() : NULL );
+                pFont    = this->CreateFontObject( it.first, m_vecFonts, pMetrics, 
+                           bEmbedd, bBold, bItalic, pszFontName, pEncoding, bSubsetting );
+            }
 
-		}
+        }
     }
     else
         pFont = (*it.first).m_pFont;
 
 #if !(defined(_WIN32) && !defined(PODOFO_NO_FONTMANAGER))
-		if (!pFont)
-	        PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
+        if (!pFont)
+            PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
 #endif             
 
     return pFont;
@@ -525,8 +477,30 @@ PdfFont* PdfFontCache::GetFont( const wchar_t* pszFontName, bool bBold, bool bIt
     PdfFont*          pFont;
     std::pair<TISortedFontList,TCISortedFontList> it;
 
-    it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), 
-                          TFontCacheElement( pszFontName, bBold, bItalic, bSymbolCharset, pEncoding ) );
+    size_t lMaxLen = wcslen(pszFontName) * 5;
+
+    if (lMaxLen == 0) 
+        PODOFO_RAISE_ERROR_INFO(ePdfError_InternalLogic, "Font name is empty");
+        
+    char* pmbFontName = static_cast<char*>(podofo_malloc(lMaxLen));
+    if (!pmbFontName)
+    {
+        PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+    }
+    if (wcstombs(pmbFontName, pszFontName, lMaxLen) == -1)
+    {
+        podofo_free(pmbFontName);
+        PODOFO_RAISE_ERROR_INFO(ePdfError_InternalLogic, "Conversion to multibyte char failed");
+    }
+
+    TFontCacheElement element;
+    element.m_bBold = bBold;
+    element.m_bItalic = bItalic;
+    element.m_pEncoding = pEncoding;
+    element.m_sFontName = pmbFontName;
+
+    it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), element );
+    
     if( it.first == it.second )
         return GetWin32Font( it.first, m_vecFonts, pszFontName, bBold, bItalic, bSymbolCharset, bEmbedd, pEncoding );
     else
@@ -544,7 +518,7 @@ PdfFont* PdfFontCache::GetFont( const LOGFONTA &logFont,
     std::pair<TISortedFontList,TCISortedFontList> it;
 
     it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), 
-		 TFontCacheElement( logFont.lfFaceName, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, logFont.lfCharSet == SYMBOL_CHARSET, pEncoding ) );
+         TFontCacheElement( logFont.lfFaceName, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, logFont.lfCharSet == SYMBOL_CHARSET, pEncoding ) );
     if( it.first == it.second )
         return GetWin32Font( it.first, m_vecFonts, logFont, bEmbedd, pEncoding );
     else
@@ -562,7 +536,7 @@ PdfFont* PdfFontCache::GetFont( const LOGFONTW &logFont,
     std::pair<TISortedFontList,TCISortedFontList> it;
 
     it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), 
-		 TFontCacheElement( logFont.lfFaceName, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, logFont.lfCharSet == SYMBOL_CHARSET, pEncoding ) );
+         TFontCacheElement( logFont.lfFaceName, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, logFont.lfCharSet == SYMBOL_CHARSET, pEncoding ) );
     if( it.first == it.second )
         return GetWin32Font( it.first, m_vecFonts, logFont, bEmbedd, pEncoding );
     else
@@ -589,12 +563,12 @@ PdfFont* PdfFontCache::GetFont( FT_Face face, bool bSymbolCharset, bool bEmbedd,
     bool bItalic = ((face->style_flags & FT_STYLE_FLAG_ITALIC) != 0);
 
     it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), 
-			   TFontCacheElement( sName.c_str(), bBold, bItalic, bSymbolCharset, pEncoding ) );
+               TFontCacheElement( sName.c_str(), bBold, bItalic, bSymbolCharset, pEncoding ) );
     if( it.first == it.second )
     {
         pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, face, bSymbolCharset );
         pFont    = this->CreateFontObject( it.first, m_vecFonts, pMetrics, 
-					   bEmbedd, bBold, bItalic, sName.c_str(), pEncoding );
+                       bEmbedd, bBold, bItalic, sName.c_str(), pEncoding );
     }
     else
         pFont = (*it.first).m_pFont;
@@ -606,13 +580,13 @@ PdfFont* PdfFontCache::GetDuplicateFontType1( PdfFont * pFont, const char* pszSu
 {
     TCISortedFontList it = m_vecFonts.begin();
 
-	std::string id = pFont->GetIdentifier().GetName();
-	id += pszSuffix;
+    std::string id = pFont->GetIdentifier().GetName();
+    id += pszSuffix;
 
     // Search if the object is a cached normal font
     while( it != m_vecFonts.end() )
     {
-		if( (*it).m_pFont->GetIdentifier() == id ) 
+        if( (*it).m_pFont->GetIdentifier() == id ) 
             return (*it).m_pFont;
 
         ++it;
@@ -629,20 +603,20 @@ PdfFont* PdfFontCache::GetDuplicateFontType1( PdfFont * pFont, const char* pszSu
     }
 
     // Create a copy of the font
-	PODOFO_ASSERT( pFont->GetFontMetrics()->GetFontType() == ePdfFontType_Type1Pfb );
-	PdfFontMetrics* pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, pFont->GetFontMetrics()->GetFilename(), pFont->GetFontMetrics()->IsSymbol() );
-	PdfFont* newFont = new PdfFontType1( static_cast<PdfFontType1 *>(pFont), pMetrics, pszSuffix, m_pParent );
+    PODOFO_ASSERT( pFont->GetFontMetrics()->GetFontType() == ePdfFontType_Type1Pfb );
+    PdfFontMetrics* pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, pFont->GetFontMetrics()->GetFilename(), pFont->GetFontMetrics()->IsSymbol() );
+    PdfFont* newFont = new PdfFontType1( static_cast<PdfFontType1 *>(pFont), pMetrics, pszSuffix, m_pParent );
     if( newFont ) 
     {
-		std::string name = newFont->GetFontMetrics()->GetFontname();
-		name += pszSuffix;
+        std::string name = newFont->GetFontMetrics()->GetFontname();
+        name += pszSuffix;
         TFontCacheElement element;
         element.m_pFont     = newFont;
         element.m_bBold     = newFont->IsBold();
         element.m_bItalic   = newFont->IsItalic();
         element.m_sFontName = name;
         element.m_pEncoding = newFont->GetEncoding();
-		  element.m_bIsSymbolCharset = pFont->GetFontMetrics()->IsSymbol();
+          element.m_bIsSymbolCharset = pFont->GetFontMetrics()->IsSymbol();
         m_vecFonts  .push_back( element );
         
         // Now sort the font list
@@ -653,65 +627,40 @@ PdfFont* PdfFontCache::GetDuplicateFontType1( PdfFont * pFont, const char* pszSu
 }
 
 PdfFont* PdfFontCache::GetFontSubset( const char* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset,
-				      const PdfEncoding * const pEncoding,
-				      const char* pszFileName )
+                      const PdfEncoding * const pEncoding,
+                      const char* pszFileName )
 {
-    PdfFont*        pFont;
+    PdfFont*        pFont = 0;
     PdfFontMetrics* pMetrics;
     std::pair<TISortedFontList,TCISortedFontList> it;
 
     // WARNING: The characters are completely ignored right now!
 
     it = std::equal_range( m_vecFontSubsets.begin(), m_vecFontSubsets.end(), 
-			   TFontCacheElement( pszFontName, bBold, bItalic, bSymbolCharset, pEncoding ) );
+               TFontCacheElement( pszFontName, bBold, bItalic, bSymbolCharset, pEncoding ) );
     if( it.first == it.second )
     {
         std::string sPath; 
-        if( pszFileName == NULL ) 
+        if( pszFileName == NULL || *pszFileName == 0) 
         {
             sPath = this->GetFontPath( pszFontName, bBold, bItalic );
             if( sPath.empty() )
             {
 #if defined(_WIN32) && !defined(PODOFO_NO_FONTMANAGER)
                 return GetWin32Font( it.first, m_vecFontSubsets, pszFontName, bBold, bItalic, bSymbolCharset, true, pEncoding, true );
-#else	    
+#else       
                 PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
                 return NULL;
 #endif // _WIN32
             }
         }
-        else
+        else {
             sPath = pszFileName;
-        
-        pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, sPath.c_str(), bSymbolCharset );
-        if( !(pMetrics && pMetrics->GetFontType() == ePdfFontType_TrueType ) )
-        {
-            PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidFontFile, "Subsetting is only supported for TrueType fonts." );
         }
         
-        PdfInputDevice          input( sPath.c_str() );
-        PdfRefCountedBuffer     buffer;
-        PdfOutputDevice         output( &buffer );
-        
-        PdfFontTTFSubset        subset( &input, pMetrics, PdfFontTTFSubset::eFontFileType_TTF );
-        PdfEncoding::const_iterator itChar
-            = pEncoding->begin();
-        int cpt = 0;
-        while( itChar != pEncoding->end() )
-        {
-            subset.AddCharacter( *itChar );
-            ++itChar;
-            cpt++;
-        }
-        subset.BuildFont( &output );
-        
-        // Delete metrics object, as it was only used so that PdfFontTTFSubset could
-        // match unicode character points to glyph indeces
-        delete pMetrics;
-
-        pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, buffer, bSymbolCharset, genSubsetBasename() );
+        pMetrics = PdfFontMetricsFreetype::CreateForSubsetting( &m_ftLibrary, sPath.c_str(), bSymbolCharset, genSubsetBasename() );
         pFont = this->CreateFontObject( it.first, m_vecFontSubsets, pMetrics, 
-                                        true, bBold, bItalic, pszFontName, pEncoding );
+                                        true, bBold, bItalic, pszFontName, pEncoding, true );
     }
     else
         pFont = (*it.first).m_pFont;
@@ -742,18 +691,18 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
 {
     LOGFONTW lf;
     
-    lf.lfHeight			= 0;
-    lf.lfWidth			= 0;
-    lf.lfEscapement		= 0;
+    lf.lfHeight         = 0;
+    lf.lfWidth          = 0;
+    lf.lfEscapement     = 0;
     lf.lfOrientation    = 0;
-    lf.lfWeight			= bBold ? FW_BOLD : 0;
-    lf.lfItalic			= bItalic;
-    lf.lfUnderline		= 0;
-    lf.lfStrikeOut		= 0;
-    lf.lfCharSet		   = bSymbolCharset ? SYMBOL_CHARSET : DEFAULT_CHARSET;
-    lf.lfOutPrecision	= OUT_DEFAULT_PRECIS;
-    lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
-    lf.lfQuality		= DEFAULT_QUALITY;
+    lf.lfWeight         = bBold ? FW_BOLD : 0;
+    lf.lfItalic         = bItalic;
+    lf.lfUnderline      = 0;
+    lf.lfStrikeOut      = 0;
+    lf.lfCharSet           = bSymbolCharset ? SYMBOL_CHARSET : DEFAULT_CHARSET;
+    lf.lfOutPrecision   = OUT_DEFAULT_PRECIS;
+    lf.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
+    lf.lfQuality        = DEFAULT_QUALITY;
     lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
     
     if (strlen(pszFontName) >= LF_FACESIZE)
@@ -763,32 +712,34 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
     //strcpy( lf.lfFaceName, pszFontName );
     /*int destLen =*/ MultiByteToWideChar (0, 0, pszFontName, -1, lf.lfFaceName, LF_FACESIZE);
 
-	 return GetWin32Font(itSorted, vecContainer, lf, bEmbedd, pEncoding, pSubsetting);
+     return GetWin32Font(itSorted, vecContainer, lf, bEmbedd, pEncoding, pSubsetting);
 }
 
 PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList & vecContainer, 
                                      const wchar_t* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset,
                                      bool bEmbedd, const PdfEncoding * const pEncoding, bool pSubsetting )
 {
-    LOGFONTW	lf;
+    LOGFONTW    lf;
     
-    lf.lfHeight			= 0;
-    lf.lfWidth			= 0;
-    lf.lfEscapement		= 0;
+    lf.lfHeight         = 0;
+    lf.lfWidth          = 0;
+    lf.lfEscapement     = 0;
     lf.lfOrientation    = 0;
-    lf.lfWeight			= bBold ? FW_BOLD : 0;
-    lf.lfItalic			= bItalic;
-    lf.lfUnderline		= 0;
-    lf.lfStrikeOut		= 0;
-    lf.lfCharSet		   = bSymbolCharset ? SYMBOL_CHARSET : DEFAULT_CHARSET;
-    lf.lfOutPrecision	= OUT_DEFAULT_PRECIS;
-    lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
-    lf.lfQuality		= DEFAULT_QUALITY;
+    lf.lfWeight         = bBold ? FW_BOLD : 0;
+    lf.lfItalic         = bItalic;
+    lf.lfUnderline      = 0;
+    lf.lfStrikeOut      = 0;
+    lf.lfCharSet           = bSymbolCharset ? SYMBOL_CHARSET : DEFAULT_CHARSET;
+    lf.lfOutPrecision   = OUT_DEFAULT_PRECIS;
+    lf.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
+    lf.lfQuality        = DEFAULT_QUALITY;
     lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
     
     pdf_long lFontNameLen = wcslen(pszFontName);
     if (lFontNameLen >= LF_FACESIZE)
         return NULL;
+    if (lFontNameLen == 0)
+        PODOFO_RAISE_ERROR_INFO(ePdfError_InternalLogic, "Font name is empty");
     
     memset(&(lf.lfFaceName), 0, LF_FACESIZE);
     wcscpy( static_cast<wchar_t*>(lf.lfFaceName), pszFontName );
@@ -797,7 +748,7 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
 }
 
 PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList & vecContainer, const LOGFONTA &logFont,
-								bool bEmbedd, const PdfEncoding * const pEncoding, bool pSubsetting)
+                                bool bEmbedd, const PdfEncoding * const pEncoding, bool pSubsetting)
 {
     char*        pBuffer = NULL;
     unsigned int nLen;
@@ -808,61 +759,59 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
     PdfFontMetrics* pMetrics;
     PdfFont*        pFont = NULL;
     try {
-		 pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, pBuffer, nLen, logFont.lfCharSet == SYMBOL_CHARSET, pSubsetting ? genSubsetBasename() : NULL );
+         pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, pBuffer, nLen, logFont.lfCharSet == SYMBOL_CHARSET, pSubsetting ? genSubsetBasename() : NULL );
         pFont    = this->CreateFontObject( itSorted, vecContainer, pMetrics, 
-			  bEmbedd, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, logFont.lfFaceName, pEncoding, pSubsetting );
+              bEmbedd, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, logFont.lfFaceName, pEncoding, pSubsetting );
     } catch( PdfError & error ) {
-        free( pBuffer );
+        podofo_free( pBuffer );
         throw error;
     }
     
-    free( pBuffer );
+    podofo_free( pBuffer );
     return pFont;
 }
 
 PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList & vecContainer, const LOGFONTW &logFont,
-								bool bEmbedd, const PdfEncoding * const pEncoding, bool pSubsetting)
+                                bool bEmbedd, const PdfEncoding * const pEncoding, bool pSubsetting)
 {
     pdf_long lFontNameLen = wcslen(logFont.lfFaceName);
     if (lFontNameLen >= LF_FACESIZE)
         return NULL;
 
-    char*        pBuffer = NULL;
-    unsigned int nLen;
-    if( !GetDataFromLPFONT( &logFont, &pBuffer, nLen ) )
-        return NULL;
-    
     pdf_long lMaxLen = lFontNameLen * 5;
-    char* pmbFontName = static_cast<char*>(malloc(lMaxLen));
+    char* pmbFontName = static_cast<char*>(podofo_malloc(lMaxLen));
     if( !pmbFontName )
     {
-        free( pBuffer );
         PODOFO_RAISE_ERROR( ePdfError_OutOfMemory );
     }
 
-    if( wcstombs( pmbFontName, logFont.lfFaceName, lMaxLen ) <= 0 )
+    if( wcstombs( pmbFontName, logFont.lfFaceName, lMaxLen ) == -1 )
     {
-        free( pBuffer );
-        free( pmbFontName );
+        podofo_free( pmbFontName );
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Conversion to multibyte char failed" );
     }
 
+    char*        pBuffer = NULL;
+    unsigned int nLen;
+    if (!GetDataFromLPFONT(&logFont, &pBuffer, nLen))
+        return NULL;
+
     PdfFontMetrics* pMetrics;
     PdfFont*        pFont = NULL;
     try {
         pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, pBuffer, nLen, logFont.lfCharSet == SYMBOL_CHARSET, pSubsetting ? genSubsetBasename() : NULL );
         pFont    = this->CreateFontObject( itSorted, vecContainer, pMetrics, 
-			  bEmbedd, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, pmbFontName, pEncoding, pSubsetting );
-        free( pmbFontName );
+              bEmbedd, logFont.lfWeight >= FW_BOLD ? true : false, logFont.lfItalic ? true : false, pmbFontName, pEncoding, pSubsetting );
+        podofo_free( pmbFontName );
         pmbFontName = NULL;
     } catch( PdfError & error ) {
-        free( pmbFontName );
+        podofo_free( pmbFontName );
         pmbFontName = NULL;
-        free( pBuffer );
+        podofo_free( pBuffer );
         throw error;
     }
     
-    free( pBuffer );
+    podofo_free( pBuffer );
     return pFont;
 }
 
@@ -922,26 +871,26 @@ std::string PdfFontCache::GetFontPath( const char* pszFontName, bool bBold, bool
 }
 
 PdfFont* PdfFontCache::CreateFontObject( TISortedFontList itSorted, TSortedFontList & rvecContainer, 
-					 PdfFontMetrics* pMetrics, bool bEmbedd, bool bBold, bool bItalic, 
-					 const char* pszFontName, const PdfEncoding * const pEncoding, bool bSubsetting ) 
+                     PdfFontMetrics* pMetrics, bool bEmbedd, bool bBold, bool bItalic, 
+                     const char* pszFontName, const PdfEncoding * const pEncoding, bool bSubsetting ) 
 {
     PdfFont* pFont;
 
     try {
         int nFlags = ePdfFont_Normal;
 
-		if ( bSubsetting )
-			nFlags |= ePdfFont_Subsetting;
-
-		if( bEmbedd )
+        if ( bSubsetting )
+            nFlags |= ePdfFont_Subsetting;
+        
+        if( bEmbedd )
             nFlags |= ePdfFont_Embedded;
-
+        
         if( bBold ) 
             nFlags |= ePdfFont_Bold;
 
         if( bItalic )
             nFlags |= ePdfFont_Italic;
-
+        
         pFont    = PdfFontFactory::CreateFontObject( pMetrics, nFlags, pEncoding, m_pParent );
 
         if( pFont ) 
@@ -952,8 +901,8 @@ PdfFont* PdfFontCache::CreateFontObject( TISortedFontList itSorted, TSortedFontL
             element.m_bItalic   = pFont->IsItalic();
             element.m_sFontName = pszFontName;
             element.m_pEncoding = pEncoding;
-				element.m_bIsSymbolCharset = pMetrics->IsSymbol();
-
+            element.m_bIsSymbolCharset = pMetrics->IsSymbol();
+            
             // Do a sorted insert, so no need to sort again
             rvecContainer.insert( itSorted, element );
         }
@@ -963,24 +912,26 @@ PdfFont* PdfFontCache::CreateFontObject( TISortedFontList itSorted, TSortedFontL
         PdfError::LogMessage( eLogSeverity_Error, "Cannot initialize font: %s\n", pszFontName ? pszFontName : "" );
         return NULL;
     }
-
+    
     return pFont;
 }
 
 const char *PdfFontCache::genSubsetBasename(void)
 {
-	int ii = 0;
-	while(ii < SUBSET_BASENAME_LEN) {
-		m_sSubsetBasename[ii]++;
-		if (m_sSubsetBasename[ii] <= 'Z') {
-			break;
-		}
+    int ii = 0;
+    while(ii < SUBSET_BASENAME_LEN)
+    {
+        m_sSubsetBasename[ii]++;
+        if (m_sSubsetBasename[ii] <= 'Z')
+        {
+            break;
+        }
 
-		m_sSubsetBasename[ii] = 'A';
-		ii++;
-	}
+        m_sSubsetBasename[ii] = 'A';
+        ii++;
+    }
 
-	return m_sSubsetBasename;
+    return m_sSubsetBasename;
 }
 
 };
