@@ -41,6 +41,7 @@
 
 #include "PdfFontFactory.h"
 
+#include <iostream>
 #include <sstream>
 
 #include <wchar.h>
@@ -57,8 +58,48 @@ namespace PoDoFo {
 #include <Carbon/Carbon.h>
 #endif
 
+struct Scoped_FT_Face {
+    Scoped_FT_Face() : ftFace(0) {
+    }
+    ~Scoped_FT_Face() {
+        if (ftFace) {
+            FT_Done_Face(ftFace);
+        }
+    }
+    FT_Face ftFace;
+};
+
+PdfFontMetricsFreetype* PdfFontMetricsFreetype::CreateForSubsetting(FT_Library* pLibrary, const char* pszFilename, bool pIsSymbol, const char* pszSubsetPrefix ) 
+{
+    Scoped_FT_Face scoped_face;
+
+    FT_Error err = FT_New_Face( *pLibrary, pszFilename, 0, &scoped_face.ftFace );
+    if (!err) {
+        FT_ULong  length = 0;
+        err = FT_Load_Sfnt_Table( scoped_face.ftFace, 0, 0, NULL, &length );
+        if (!err) {
+            PdfRefCountedBuffer buffer(length);
+            err = FT_Load_Sfnt_Table( scoped_face.ftFace, 0, 0, reinterpret_cast<FT_Byte*>(buffer.GetBuffer()), &length );
+            if (!err) {
+                return new PdfFontMetricsFreetype( pLibrary, buffer, pIsSymbol, pszSubsetPrefix );
+            }
+        }
+        // throw an exception
+        PdfError::LogMessage( eLogSeverity_Critical, "FreeType returned the error %i when calling FT_Load_Sfnt_Table for font %s.", 
+                              err, pszFilename );
+        PODOFO_RAISE_ERROR( ePdfError_FreeType );
+    }
+    else {
+        // throw an exception
+        PdfError::LogMessage( eLogSeverity_Critical, "FreeType returned the error %i when calling FT_New_Face for font %s.", 
+                              err, pszFilename );
+        PODOFO_RAISE_ERROR( ePdfError_FreeType );
+    }
+    return 0;
+}
+
 PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary, const char* pszFilename, 
-															   bool pIsSymbol, const char* pszSubsetPrefix )
+                                                bool pIsSymbol, const char* pszSubsetPrefix )
     : PdfFontMetrics( PdfFontMetrics::FontTypeFromFilename( pszFilename ),
                       pszFilename, pszSubsetPrefix ),
       m_pLibrary( pLibrary ),
@@ -79,7 +120,7 @@ PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary, const char
 
 PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary, 
                                                 const char* pBuffer, unsigned int nBufLen,
-																bool pIsSymbol,
+                                                                bool pIsSymbol,
                                                 const char* pszSubsetPrefix )
     : PdfFontMetrics( ePdfFontType_Unknown, "", pszSubsetPrefix ),
       m_pLibrary( pLibrary ),
@@ -94,7 +135,7 @@ PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary,
 
 PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary, 
                                                 const PdfRefCountedBuffer & rBuffer,
-																bool pIsSymbol,
+                                                                bool pIsSymbol,
                                                 const char* pszSubsetPrefix ) 
     : PdfFontMetrics( ePdfFontType_Unknown, "", pszSubsetPrefix ),
       m_pLibrary( pLibrary ),
@@ -107,7 +148,7 @@ PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary,
 
 PdfFontMetricsFreetype::PdfFontMetricsFreetype( FT_Library* pLibrary, 
                                                 FT_Face face, 
-																bool pIsSymbol,
+                                                                bool pIsSymbol,
                                                 const char* pszSubsetPrefix  )
     : PdfFontMetrics( ePdfFontType_TrueType, 
                       // Try to initialize the pathname from m_face
@@ -135,12 +176,15 @@ PdfFontMetricsFreetype::~PdfFontMetricsFreetype()
 
 void PdfFontMetricsFreetype::InitFromBuffer(bool pIsSymbol)
 {
-    FT_Error error = FT_New_Memory_Face( *m_pLibrary, 
-                                         reinterpret_cast<const unsigned char*>(m_bufFontData.GetBuffer()), 
-                                         static_cast<long>(m_bufFontData.GetSize()), 0, &m_pFace );
+    FT_Open_Args openArgs;
+    memset(&openArgs, 0, sizeof(openArgs));
+    openArgs.flags = FT_OPEN_MEMORY;
+    openArgs.memory_base = reinterpret_cast<FT_Byte*>(m_bufFontData.GetBuffer()), 
+    openArgs.memory_size = static_cast<FT_Long>(m_bufFontData.GetSize());
+    FT_Error error = FT_Open_Face( *m_pLibrary, &openArgs, 0, &m_pFace ); 
     if( error ) 
     {
-        PdfError::LogMessage( eLogSeverity_Critical, "FreeType return edthe error %i when calling FT_New_Face for a buffered font.", error );
+        PdfError::LogMessage( eLogSeverity_Critical, "FreeType returned the error %i when calling FT_New_Face for a buffered font.", error );
         PODOFO_RAISE_ERROR( ePdfError_FreeType );
     }
     else
@@ -168,7 +212,7 @@ void PdfFontMetricsFreetype::InitFromFace(bool pIsSymbol)
     m_dStrikeOutPosition  = 0.0;
     m_dStrikeOutThickness = 0.0;
     m_fFontSize           = 0.0f;
-	 m_bSymbol = pIsSymbol;
+     m_bSymbol = pIsSymbol;
     m_bIsBold = false;
     m_bIsItalic = false;
 
@@ -178,48 +222,49 @@ void PdfFontMetricsFreetype::InitFromFace(bool pIsSymbol)
         m_dPdfDescent = m_pFace->descender * 1000.0 / m_pFace->units_per_EM;
         m_bIsBold = (m_pFace->style_flags & FT_STYLE_FLAG_BOLD) != 0;
         m_bIsItalic = (m_pFace->style_flags & FT_STYLE_FLAG_ITALIC) != 0;
-    }
 
-    // Try to get a unicode charmap
-    FT_Select_Charmap( m_pFace, pIsSymbol ? FT_ENCODING_MS_SYMBOL : FT_ENCODING_UNICODE );
+        // Try to get a unicode charmap
+        FT_Select_Charmap( m_pFace, pIsSymbol ? FT_ENCODING_MS_SYMBOL : FT_ENCODING_UNICODE );
 
-    // Try to determine if it is a symbol font
-    for( int c=0;c<m_pFace->num_charmaps;c++ ) 
-    {  
-        FT_CharMap charmap = m_pFace->charmaps[c]; 
+        // Try to determine if it is a symbol font
+        for( int c=0; c<m_pFace->num_charmaps; c++ ) 
+        {  
+            FT_CharMap charmap = m_pFace->charmaps[c]; 
 
-        if( charmap->encoding == FT_ENCODING_MS_SYMBOL ) 
-        {
-            m_bSymbol = true;
-            FT_Set_Charmap( m_pFace, charmap );
-            break;
+            if( charmap->encoding == FT_ENCODING_MS_SYMBOL ) 
+            {
+                m_bSymbol = true;
+                FT_Set_Charmap( m_pFace, charmap );
+                break;
+            }
+            // TODO: Also check for FT_ENCODING_ADOBE_CUSTOM and set it?
         }
-        // TODO: Also check for FT_ENCODING_ADOBE_CUSTOM and set it?
-    }
     
-    // we cache the 256 first width entries as they 
-    // are most likely needed quite often
-    m_vecWidth.clear();
-    m_vecWidth.reserve( PODOFO_WIDTH_CACHE_SIZE );
-    for( unsigned int i=0;i<PODOFO_WIDTH_CACHE_SIZE;i++ )
-    {
-        if( i < PODOFO_FIRST_READABLE || !m_pFace )
-            m_vecWidth.push_back( 0.0  );
-        else
+        // we cache the 256 first width entries as they
+        // are most likely needed quite often
+        m_vecWidth.clear();
+        m_vecWidth.reserve( PODOFO_WIDTH_CACHE_SIZE );
+        for( unsigned int i=0; i < PODOFO_WIDTH_CACHE_SIZE; i++ )
         {
-            int index = i;
-            // Handle symbol fonts
-            if( m_bSymbol ) 
+            if( i < PODOFO_FIRST_READABLE || !m_pFace )
+                m_vecWidth.push_back( 0.0  );
+            else
             {
-                index = index | 0xf000;
-            }
+                int index = i;
+                // Handle symbol fonts
+                if( m_bSymbol ) 
+                {
+                    index = index | 0xf000;
+                }
 
-            if( !FT_Load_Char( m_pFace, index, FT_LOAD_NO_SCALE | FT_LOAD_NO_BITMAP ) )  // | FT_LOAD_NO_RENDER
-            {
-                //m_vecWidth.push_back( 0.0  );
-                //continue;
+                if( FT_Load_Char( m_pFace, index, FT_LOAD_NO_SCALE | FT_LOAD_NO_BITMAP) == 0 )  // | FT_LOAD_NO_RENDER
+                {
+                    m_vecWidth.push_back( static_cast<double>(m_pFace->glyph->metrics.horiAdvance) * 1000.0 / m_pFace->units_per_EM );
+                    continue;
+                }
+                
+                m_vecWidth.push_back( 0.0  );
             }
-            m_vecWidth.push_back( static_cast<double>(m_pFace->glyph->metrics.horiAdvance) * 1000.0 / m_pFace->units_per_EM );
         }
     }
 
@@ -261,7 +306,7 @@ const char* PdfFontMetricsFreetype::GetFontname() const
     return s ? s : "";
 }
 
-void PdfFontMetricsFreetype::GetWidthArray( PdfVariant & var, unsigned int nFirst, unsigned int nLast ) const
+void PdfFontMetricsFreetype::GetWidthArray( PdfVariant & var, unsigned int nFirst, unsigned int nLast, const PdfEncoding* pEncoding ) const
 {
     unsigned int  i;
     PdfArray  list;
@@ -273,18 +318,27 @@ void PdfFontMetricsFreetype::GetWidthArray( PdfVariant & var, unsigned int nFirs
 
     for( i=nFirst;i<=nLast;i++ )
     {
-        if( i < PODOFO_WIDTH_CACHE_SIZE )
+        if( i < PODOFO_WIDTH_CACHE_SIZE && pEncoding == NULL )
             list.push_back( PdfVariant( m_vecWidth[i] ) );
         else
         {
-            if( !FT_Load_Char( m_pFace, i, FT_LOAD_NO_SCALE | FT_LOAD_NO_BITMAP ) )  // | FT_LOAD_NO_RENDER
+            if (pEncoding != NULL)
             {
-                //PODOFO_RAISE_ERROR( ePdfError_FreeType );
-                list.push_back( PdfVariant( 0.0 ) );
+                unsigned short shCode = pEncoding->GetCharCode(i);
+#ifdef PODOFO_IS_LITTLE_ENDIAN
+                shCode = ((shCode & 0x00FF) << 8) | ((shCode & 0xFF00) >> 8);
+#endif
+                list.push_back( PdfVariant( (pdf_int64)this->GetGlyphWidth(this->GetGlyphId(shCode)) ) );
                 continue;
             }
-
-            list.push_back( PdfVariant( m_pFace->glyph->metrics.horiAdvance * 1000.0 / m_pFace->units_per_EM ) );
+            else if( FT_Load_Char( m_pFace, i, FT_LOAD_NO_SCALE | FT_LOAD_NO_BITMAP ) == 0 )  // | FT_LOAD_NO_RENDER
+            {
+                // zero return code is success!
+                list.push_back( PdfVariant( m_pFace->glyph->metrics.horiAdvance * 1000.0 / m_pFace->units_per_EM ) );
+                continue;
+            }
+            //PODOFO_RAISE_ERROR( ePdfError_FreeType );
+            list.push_back( PdfVariant( 0.0 ) );
         }
     }
 
@@ -309,7 +363,7 @@ double PdfFontMetricsFreetype::GetGlyphWidth( int nGlyphId ) const
 
 double PdfFontMetricsFreetype::GetGlyphWidth( const char* pszGlyphname ) const
 {
-	return GetGlyphWidth( FT_Get_Name_Index( m_pFace, const_cast<char *>(pszGlyphname) ) );
+    return GetGlyphWidth( FT_Get_Name_Index( m_pFace, const_cast<char *>(pszGlyphname) ) );
 }
 
 void PdfFontMetricsFreetype::GetBoundingBox( PdfArray & array ) const
@@ -339,7 +393,6 @@ double PdfFontMetricsFreetype::UnicodeCharWidth( unsigned short c ) const
     FT_Error ftErr;
     double   dWidth = 0.0;
 
-
     if( static_cast<int>(c) < PODOFO_WIDTH_CACHE_SIZE ) 
     {
         dWidth = m_vecWidth[static_cast<unsigned int>(c)];
@@ -402,7 +455,7 @@ double PdfFontMetricsFreetype::GetUnderlinePosition() const
 // -----------------------------------------------------
 double PdfFontMetricsFreetype::GetStrikeOutPosition() const
 {
-	return m_dStrikeOutPosition * this->GetFontSize();
+    return m_dStrikeOutPosition * this->GetFontSize();
 }
 
 // -----------------------------------------------------
