@@ -46,16 +46,22 @@ PdfFontMetricsObject::PdfFontMetricsObject( PdfObject* pFont, PdfObject* pDescri
     : PdfFontMetrics( ePdfFontType_Unknown, "", NULL ),
       m_pEncoding( pEncoding ), m_dDefWidth(0.0)
 {
+    m_missingWidth = NULL;
+
     const PdfName & rSubType = pFont->GetDictionary().GetKey( PdfName::KeySubtype )->GetName();
 
     // OC 15.08.2010 BugFix: /FirstChar /LastChar /Widths are in the Font dictionary and not in the FontDescriptor
 	if ( rSubType == PdfName("Type1") || rSubType == PdfName("Type3") || rSubType == PdfName("TrueType") ) {
         if ( pDescriptor ) {
-            m_sName        = pDescriptor->GetIndirectKey( "FontName" )->GetName();
-            m_bbox         = pDescriptor->GetIndirectKey( "FontBBox" )->GetArray();
+            if (pDescriptor->GetDictionary().HasKey( "FontName" ))
+                m_sName        = pDescriptor->GetIndirectKey( "FontName" )->GetName();
+            if (pDescriptor->GetDictionary().HasKey( "FontBBox" ))
+                m_bbox         = pDescriptor->GetIndirectKey( "FontBBox" )->GetArray();
         } else {
-            m_sName        = pFont->GetIndirectKey( "Name" )->GetName();
-            m_bbox         = pFont->GetIndirectKey( "FontBBox" )->GetArray();
+            if (pFont->GetDictionary().HasKey( "Name" ))
+                m_sName        = pFont->GetIndirectKey( "Name" )->GetName();
+            if (pFont->GetDictionary().HasKey( "FontBBox" ))
+                m_bbox         = pFont->GetIndirectKey( "FontBBox" )->GetArray();
         }
         if (pFont->GetDictionary().HasKey( "FontMatrix" )) {
             // Type3 fonts have a custom FontMatrix
@@ -81,8 +87,8 @@ PdfFontMetricsObject::PdfFontMetricsObject( PdfObject* pFont, PdfObject* pDescri
             if( widths == NULL ) 
             {
                 PODOFO_RAISE_ERROR_INFO( ePdfError_NoObject, "Font object defines neither Widths, nor MissingWidth values!" );
-                m_missingWidth = widths;
             }
+            m_missingWidth = widths;
         }
 	} else if ( rSubType == PdfName("CIDFontType0") || rSubType == PdfName("CIDFontType2") ) {
 		PdfObject *pObj = pDescriptor->GetIndirectKey( "FontName" );
@@ -193,8 +199,7 @@ double PdfFontMetricsObject::CharWidth( unsigned char c ) const
     {
         double dWidth = m_width[c - m_nFirst].GetReal();
         
-        return dWidth * static_cast<double>(this->GetFontSize() * this->GetFontScale() / 100.0) * m_matrix.front().GetReal() +
-            static_cast<double>( this->GetFontSize() * this->GetFontScale() / 100.0 * this->GetFontCharSpace() / 100.0);
+        return (dWidth * m_matrix.front().GetReal() * this->GetFontSize() + this->GetFontCharSpace()) * this->GetFontScale() / 100.0;
 
     }
 
@@ -210,10 +215,8 @@ double PdfFontMetricsObject::UnicodeCharWidth( unsigned short c ) const
         && c - m_nFirst < static_cast<int>(m_width.GetSize()) )
     {
         double dWidth = m_width[c - m_nFirst].GetReal();
-
-        return dWidth * static_cast<double>(this->GetFontSize() * this->GetFontScale() / 100.0) * m_matrix.front().GetReal() +
-            static_cast<double>( this->GetFontSize() * this->GetFontScale() / 100.0 * this->GetFontCharSpace() / 100.0);
-
+        
+        return (dWidth * m_matrix.front().GetReal() * this->GetFontSize() + this->GetFontCharSpace()) * this->GetFontScale() / 100.0;
     }
 
     if( m_missingWidth != NULL )
@@ -222,7 +225,7 @@ double PdfFontMetricsObject::UnicodeCharWidth( unsigned short c ) const
         return m_dDefWidth;
 }
 
-void PdfFontMetricsObject::GetWidthArray( PdfVariant & var, unsigned int, unsigned int ) const
+void PdfFontMetricsObject::GetWidthArray( PdfVariant & var, unsigned int, unsigned int, const PdfEncoding* ) const
 {
     var = m_width;
 }
