@@ -37,6 +37,8 @@
 
 #include "base/PdfDictionary.h"
 #include "base/PdfEncoding.h"
+#include "PdfFontMetricsBase14.h"
+#include "base/PdfArray.h"
 
 namespace PoDoFo {
 
@@ -69,9 +71,26 @@ void PdfFontType1Base14::InitBase14Font( PdfFontMetrics* pMetrics )
 {
     PdfVariant    var;
     
+    if( !m_pEncoding )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
     this->GetObject()->GetDictionary().AddKey( PdfName::KeySubtype, PdfName("Type1"));
     this->GetObject()->GetDictionary().AddKey("BaseFont", PdfName( pMetrics->GetFontname() ) );
-		 
+
+    PdfObject *pWidth = this->GetObject()->GetOwner()->CreateObject();
+    if( !pWidth )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+    m_pMetrics->GetWidthArray( *pWidth, m_pEncoding->GetFirstChar(), m_pEncoding->GetLastChar(), m_pEncoding );
+
+    this->GetObject()->GetDictionary().AddKey("Widths", pWidth->Reference() );
+    this->GetObject()->GetDictionary().AddKey("FirstChar", PdfVariant( static_cast<pdf_int64>(m_pEncoding->GetFirstChar()) ) );
+    this->GetObject()->GetDictionary().AddKey("LastChar", PdfVariant( static_cast<pdf_int64>(m_pEncoding->GetLastChar()) ) );
+
     m_pEncoding->AddToDictionary( this->GetObject()->GetDictionary() ); // Add encoding key
 //	pDescriptor->GetDictionary().AddKey( "FontName", this->GetBaseFont() );
     //pDescriptor->GetDictionary().AddKey( "FontWeight", (long)m_pMetrics->Weight() );
